/**
        Linked Lists using double pointer technique
        @author David Yue
 */

#include <stdio.h>
#include <stdlib.h>

struct linkedlist_node {
        int item;
        struct linkedlist_node *next;
};

struct linkedlist_node *create_node (int value)
{
        struct linkedlist_node *new_node = malloc (sizeof (struct linkedlist_node));

        if (!new_node)
                return NULL;

        new_node->item = value;
        new_node->next = NULL;

        return new_node;
}

void insert_at_end (struct linkedlist_node *list, struct linkedlist_node *node)
{
        // Assumes list != NULL
        while (list->next) {
                list = list->next;
        }
        list->next = node;
}

void insert_at_begin (struct linkedlist_node **list, struct linkedlist_node *node)
{
        node->next = *list;
        *list = node;
}

void insert_ordered (struct linkedlist_node **list, struct linkedlist_node *node)
{
        while (*list && (*list)->item < node->item)
                list = &((*list)->next);

        node->next = *list;
        *list = node;
}

void delete_item (struct linkedlist_node **list, int value)
{
        while (*list && (*list)->item != value)
                list = &((*list)->next);

        struct linkedlist_node *to_del = *list;

        if (!to_del)
                return;

        *list = to_del->next;

        free (to_del);
}

int main (int argc, char **argv)
{
        int node_values[3] = { 1, 7, 10 };

        struct linkedlist_node *head = create_node (0);

        for (int i = 0; i < 3; i++) {
                insert_at_end (head, create_node (node_values[i]));
        }

        insert_ordered (&head, create_node (5));
        insert_ordered (&head, create_node (-1));

        while (head != NULL) {
                printf ("Node %d\n", head->item);
                head = head->next;
        }

        return 0;
}